<?php
/**
 * CDataFilter class file.
 *
 * @author Seb <serebrov@algo-rithm.com>, Algo-rithm
 *
 * @version 0.1
 *
 * @desc CDataFilter represents information relevant to list (table) data filtration and search.
 *
 * When the data needs to be filtered or searched by some fields, we can use CDataFilter to
 * represent information about filter and search controls
 * These information can be passed to CDataFilterWidget to render
 * filter controls.
 *
 * Note. This class has no relation to yii's CFilter class.
 *
 * Not two filter types are supported:
 *   FILTER_DROPDOWN - simple dropdown list, when option is selected, data filtration
 *                     performed
 *   FILTER_SEARCH - dropdown list with field names and a text filed to enter
 *                   search criteria, when Search button is clicked, search is
 *                   performed
 *
 * Class receives model name, model should support following methods:
 *  - getDataFilterOptions($filterName)
 *  - getDataFilterSearchFields($filterName)
 *  - applyDataFilterCriteria(&$criteria, $filterName, $filterValue)
 *  - applyDataSearchCriteria(&$criteria, $filterName, $searchField, $serachValue)
 *
 * ************************ Using ***********************************
 * *** Controller ***
 * 
 *  public function actionAdmin()
 *	{
 *		$this->processAdminCommand();
 *
 *		$criteria=new CDbCriteria;
 *
 *      $filters = new CDataFilter(User::model());
 *      // add search by user's fields (fields list taken from User::getDataFilterSearchFields())
 *      $filters->addFilter(CDataFilter::FILTER_SEARCH,   'userFieldsSearch', '');
 *      // add dropdown filter by user's school (schools list taken from User::getDataFilterOptions())
 *      $filters->addFilter(CDataFilter::FILTER_DROPDOWN, 'schoolFilter', 'School');
 *      // add dropdown filter by user's class (classes list taken from User::getDataFilterOptions())
 *      $filters->addFilter(CDataFilter::FILTER_DROPDOWN, 'classFilter', 'Class');
 *
 *      $filters->applyCriteria($criteria);
 *
 *      $pages=new CPagination(User::model()->count($criteria));
 *      ...
 *		$sort=new CSort('User');
 *
 *      $models=User::model()->findAll($criteria);
 *
 *      $params = array(
 *		      	'models'=>$models,
 *   			'pages'=>$pages,
 *	      		'sort'=>$sort,
 *              'filters'=>$filters
 *	    );
 *
 *   	if (Yii::app()->request->isAjaxRequest) {
 *          // support for ajax filtering
 *          // "adminList" is parially rendered inside "admin" template
 *          // and contains data table
 *       	$this->renderPartial('adminList', $params);
 *       } else {
 *   		$this->render('admin', $params);
 *       }
 *	}
 *
 * *** Model ***
 *
 * // Filter name is passed
 * // Filter values are available from $_GET array
 * // Method should return array with options for dropdown list
 * // This method only used for FILTER_DROPDOWN filters
 * public function getDataFilterOptions($filterName)
 *   {
 *       switch ($filterName) {
 *           case 'schoolFilter':
 *              $schools = School::model()->findAll();
 *              return CHtml::listData($schools, 'id', 'name');
 *           case 'classFilter':
 *              // This filter is dependent from School - when school is
 *              // selected, only selected School's classes should be shown in dropdown
 *              $schoolId = $_GET['schoolFilter'];
 *              $criteria = new CDbCriteria;
 *              if (isset($schoolId) && $schoolId > 0) {
 *                  $criteria->condition = ' schools_id='.$schoolId.' ';
 *              }
 *              $classes = Classes::model()->findAll($criteria);
 *              return CHtml::listData($classes, 'id', 'name'); 
 *           case 'userTypeFilter':
 *              $userTypes = UserType::model()->findAll();
 *              return CHtml::listData($userTypes, 'id', 'name');
 *       }
 *   }
 *
 *   // Filter name is passed
 *   // Method should return array with search fileds for dropdown list
 *   // This method only used for FILTER_SEARCH filters
 *   public function getDataFilterSearchFields($filterName)
 *   {
 *       switch ($filterName) {
 *           case 'userFieldsSearch':
 *               return array(
 *                   'user_id'=>'User ID',
 *                   'lname'=>'Last Name',
 *                   'fname'=>'First Name',
 *                   'username'=>'User Name',
 *               );
 *       }
 *   }
 *
 *
 *   // Criteria is passed - method should apply condition or other criteria
 *   // parameters to do the actual data filtering
 *   // This method used for FILTER_DROPDOWN and FILTER_SEARCH filters
 *  public function applyDataFilterCriteria(&$criteria, $filterName, $filterValue)
 *   {
 *       if($filterName == 'schoolFilter') {
 *           $localCriteria = new CDbCriteria;
 *           $localCriteria->condition = ' schools_id='.$filterValue.' ';
 *           $criteria->mergeWith($localCriteria);
 *       }
 *
 *       if($filterName == 'classFilter') {
 *           $localCriteria = new CDbCriteria;
 *           ...
 *           $criteria->mergeWith($localCriteria);
 *       }
 *
 *       if ($filterName == 'userTypeFilter') {
 *           $localCriteria = new CDbCriteria;
 *           $localCriteria->condition = ' user_types_id='. $filterValue .' ';
 *           $criteria->mergeWith($localCriteria);
 *       }
 *   }
 *
 *   public function applyDataSearchCriteria(&$criteria, $filterName, $searchField, $searchValue)
 *   {
 *       if($filterName == 'userFieldsSearch') {
 *           $localCriteria = new CDbCriteria;
 *           $localCriteria->condition = ' '.$searchField.' LIKE "%' .$searchValue.'%" ';
 *           $criteria->mergeWith($localCriteria);
 *       }
 *   }
 *
 *  *** View ***
 *  <?php $this->widget('CDataFilterWidget',array('filters'=>$filters)); ?>
 *
 *  OR
 *  // enable ajax filtering / searching - '#updateData' selector will be replaced
 *  <?php $this->widget('CDataFilterWidget',array('filters'=>$filters,
 *        'ajaxMode'=>true, 'updateSelector'=>'#updateData')); ?>
 *  
 */

class CDataFilter extends CComponent
{   
    public $filters = array();
    public $model;

    /**
     * Constructor.
     * @model to add filter to.
     */
    public function __construct($model)
    {
        $this->model = $model;
    }

    /**
     *
     * @param <int> $filterType - FILTER_DROPDOWN or FILTER_SEARCH
     * @param <string> $filterName - filter name
     * @param <type> $filterDisplayName - filter label
     * @param <type> $emptyValue - empty filter value (user only for FILTER_DROPDOWN),
     *               if set to '' - no empty value
     */
    public function addFilter($filter)
    {
        $this->filters[] = $filter;
    }

    public function applyCriteria(&$criteria)
    {
        foreach ($this->filters as $filter) {
            if ($filter->hasValue())
                $filter->applyCriteria($this->model, $criteria);
        }
    }    

    /**
     * Returns filters count
     */
    public function getCount()
    {
        return count($this->filters);
    }

    public function hasValues()
    {
        foreach ($this->filters as $filter) {
            if ($filter->hasValue()) return true;
        }
        return false;

    }

    public static function getCondition($fieldName, $fieldValue)
    {
        if (strtolower($fieldValue) === 'null') {
            return ' '.$fieldName.' is null';
        } else {
            return ' '.$fieldName.' ='.$fieldValue.' ';
        }
    }


}